!***********************************************************************
      program asifem 		!ASI-Gauss解析コード ver.1.3
!*** ASIFEMのメインプログラム
!***********************************************************************
      include 'param.dat'
      dimension coord(mnode+melm,3),dlx(mnode),dly(mnode),
     .           dlz(mnode),el(melm),elori(melm),emate(mmate,mmd),
     .           eqs(melm),eqsout(melm,2),esect(msect,msd),f(mtdf),
     .           fout(mtdf),ind(mnode),ir(melm),
     .           mc(mnc,2),ncr(melm),ne(melm,mdim_ne),
     .           net(mtdf),nte(mtdf),pmx(melm),pmy(melm),pn(melm),
     .           ss(melm),t(melm,3,3),ta(melm,mmtx,mmtx),
     .           tacc(mtdf),te(melm,mdim),tru(melm,3,3),ts(melm,mdim),
     .           tsc(melm,mdim),tsg(melm,mdim),tsl(melm,mdim),
     .           tsr(melm,mdim),tu(mtdf),tuout(mnode,3),tvel(mtdf),
     .           x(maxlena), dltx(mnode),dlty(mnode),dltz(mnode)
!
!*** ファイルを開く *** 
      open(ii5,status='old',file='input.txt')
      open(ii6,file='post.out')
      open(ii7,file='eqs.out')
      open(ii9,file='output.txt')
!
!*** パラメータの初期化と入力データの読み込み ***
      call inicon(fout,tacc,te,ts,tsc,tsg,tsl,tsr,tu,tuout,tvel,x)
      call input(beta_n,dalfa,dbeta,delta_n,dtime,inielm,nc,ndnod,
     .           nelm,nfree,nlstart,nlstep,nmate,nmetd,nmm,node,
     .           nout,nsd,nsect,nstage,ntdf,coord,dlx,dly,dlz,
     .           emate,esect,ind,ir,mc,ne,pmx,pmy,pn,ss,tuout,
     .           nstr,neg,nerr,neps,dltx,dlty,dltz)
!
!*** 境界条件の設定およびCG法のためのパラメータ初期化 ***
      call bdcon(nc,nfree,ntdf,mc,net,nte)
      call cginit(nelm,nfree,node,ntdf,ne,net)
!
!*** 計算ループ（istage=0: 自重解析のステップ）***
      time=0.0d0
      do istage=0,nstage
!
!*** 座標変換マトリックスの作成 ***
        call trans1(istage,nelm,coord,el,elori,ne,t,ta,tru,nerr)
!
!*** 全体質量マトリックスの作成 ***
        call tmasm(nelm,nmm,nfree,ntdf,elori,emate,esect,ne,ta)
!
!*** 全体剛性マトリックスの作成 ***
        call tstfns(nelm,nfree,ntdf,el,elori,emate,esect,ncr,ne,pmx,pmy,
     .             pn,ss,ta,ts)
!
!*** 外力と残差力の作成 ***
        call force(istage,ndnod,nelm,nlstart,nlstep,ntdf,dlx,dly,dlz,
     .             elori,emate,esect,f,ind,ne,nte,neg,dltx,dlty,dltz)
        call resid(nelm,nfree,ntdf,el,f,fout,ne,net,ss,ta,ts,esect)
!
!*** 連立方程式を解き，変位，ひずみ，断面力等を計算 ***
        call solve(beta_n,dalfa,dbeta,delta_n,dtime,istage,nelm,
     .             nfree,nmetd,node,nsd,coord,el,emate,eqs,eqsout,
     .             esect,f,ir,ncr,ne,net,nte,pmx,pmy,pn,ss,t,ta,tacc,
     .             te,tru,ts,tsc,tsg,tsl,tsr,tu,tuout,tvel,x,nstr,neg,
     .             elori,neps)
!
!*** 数値積分点と応力評価点のシフト ***
        call scont(inielm,nelm,nmetd,ir,ncr,ne,ss,ts,tsc,tsg,tsl,tsr)
!
!*** 結果の出力 ***
        call output(inielm,iout,istage,node,nout,nstage,time,eqsout,
     .              ne,tuout,tu,ts,nelm)
!
!*** 経過時間の計算 ***
        if(nsd==2) time=time+dtime
      enddo
!
      end
!***********************************************************************
!*************************** Subroutines *******************************
!***********************************************************************
!
!***********************************************************************
      subroutine bdcon(nc,nfree,ntdf,mc,net,nte)
!*** 境界条件の設定
!***********************************************************************
      include 'param.dat'
      dimension mc(mnc,2),net(mtdf),nte(mtdf)
!
!*** 自由度番号対応表の初期化 ***
      do i=1,ntdf
        net(i)=0
        nte(i)=0
      enddo
!
!*** 自由度番号対応表の作成 ***
!
!*** 拘束自由度について ***
      do i=1,nc
        i1=6*(mc(i,1)-1)+mc(i,2)
        nte(nfree+i)=i1
        net(i1)=nfree+i
      enddo
!
!*** 自由な自由度について ***
      k=1
      do i=1,ntdf
        if(net(i)==0) then
          nte(k)=i
          net(i)=k
          k=k+1
        endif
      enddo
!
      return
      end
!***********************************************************************
      subroutine bmake(ielm,s,bmatx,el,ta,esect,ne)
!*** [B]マトリックス（一般化ひずみ－節点変位マトリックス）の作成
!***********************************************************************
      include 'param.dat'
      dimension bmatx(mdim,mmtx),el(melm),ta(melm,mmtx,mmtx),
     .           ba(mdim,mmtx),ne(melm,mdim_ne),esect(msect,msd)
!
!*** [B]マトリックス成分の初期化 ***
      do i=1,mdim
        do j=1,mmtx
          ba(i,j)=0.0d0
        enddo
      enddo
!
!*** 線形Timoshenkoはり要素の[B]マトリックス（(4.21)式参照）***
      bc=1.0d0/el(ielm)
      ba(1,4)=bc
      ba(1,10)=-bc
      ba(2,5)=-bc
      ba(2,11)=bc
      ba(3,3)=-bc
      ba(3,9)=bc
      ba(4,6)=-bc
      ba(4,12)=bc
      ba(5,1)=-bc
      ba(5,5)=-(1.0d0-s)/2.0d0
      ba(5,7)=bc
      ba(5,11)=-(1.0d0+s)/2.0d0
      ba(6,2)=-bc
      ba(6,4)=(1.0d0-s)/2.0d0
      ba(6,8)=bc
      ba(6,10)=(1.0d0+s)/2.0d0
!
!*** 座標の変換 ***
  100 continue
      do i=1,6
        do j=1,12
          sum=0.0d0
          do k=1,12
            sum=sum+ba(i,k)*ta(ielm,k,j)
          enddo
          bmatx(i,j)=sum
        enddo
      enddo
!
      return
      end
!***********************************************************************
      subroutine dmake(am,ielm,dmatx,emate,esect,fd,ncr,ne,pmx,pmy,
     .                 pn,q,ts)
!*** [D]マトリックス（一般化断面力－一般化ひずみマトリックス）の作成
!***********************************************************************
      include 'param.dat'
      dimension dmatx(mdim,mdim),emate(mmate,mmd),esect(msect,msd),
     .          fd(3),ncr(melm),ne(melm,mdim_ne),pmx(melm),pmy(melm),
     .          pn(melm),q(3),ts(melm,mdim)
!
!*** [D]マトリックス成分の初期化 ***
      do i=1,mdim
        do j=1,mdim
          dmatx(i,j)=0.0d0
        enddo
      enddo
!
!*** 線形Timoshenkoはり要素の[D]マトリックス（(4.25)式参照） ***
      i4=ne(ielm,4)
      G=emate(i4,1)/(2.0d0*(1.0d0+emate(i4,2)))
      dmatx(1,1)=emate(i4,1)*esect(i4,2)  		! EI_x x軸回り曲げ剛性
      dmatx(2,2)=emate(i4,1)*esect(i4,3)  		! EI_y y軸回り曲げ剛性
      dmatx(3,3)=emate(i4,1)*esect(i4,1)   		! EA 軸剛性
      dmatx(4,4)=G*esect(i4,4)            		! GJ ねじり剛性
      dmatx(5,5)=esect(i4,7)*G*esect(i4,1)  		! α_x GA x軸方向せん断剛性
      dmatx(6,6)=esect(i4,8)*G*esect(i4,1)  		! α_y GA y軸方向せん断剛性
!
!*** 塑性域における[D]マトリックスの作成（(6.26)式参照）***
      if(ncr(ielm)>0) then
          fd(1)=2.0d0*ts(ielm,1)/pmx(ielm)/pmx(ielm)
          fd(2)=2.0d0*ts(ielm,2)/pmy(ielm)/pmy(ielm)
          fd(3)=2.0d0*ts(ielm,3)/pn(ielm)/pn(ielm)
!
        am=0.0d0
        do i=1,3
          am=am+dmatx(i,i)*fd(i)*fd(i)
        enddo
!
          whard=emate(i4,1)*emate(i4,4)/(emate(i4,1)-emate(i4,4))
          am=am+whard/esect(i4,1)
!
        do i=1,3
          q(i)=dmatx(i,i)*fd(i)
        enddo
        do i=1,3
          do j=1,3
            dmatx(i,j)=dmatx(i,j)-q(i)*q(j)/am
          enddo
        enddo
      endif
!
      return
      end
!***********************************************************************
      subroutine elmass(ielm,nmm,elori,emate,emmat,esect,ne,ta)
!*** 要素質量マトリックス[M_E]の作成
!***********************************************************************
      include 'param.dat'
      dimension elori(melm),emate(mmate,mmd),emmat(mmtx,mmtx),
     .           esect(msect,msd),ne(melm,mdim_ne),ta(melm,mmtx,mmtx),
     .           emt(mmtx,mmtx)
!
!*** [M_E]マトリックス成分の初期化 ***
      do i=1,mmtx
        do j=1,mmtx
          emmat(i,j)=0.0d0
        enddo
      enddo
!
!*** 線形Timoshenkoはり要素の分布質量マトリックス（(4.26)式参照）***
        i4=ne(ielm,4)
        arl=emate(i4,5)*esect(i4,1)*elori(ielm)/(6.0d0*9.8d+03)	! ρAl/6/9800
        brl=emate(i4,5)*esect(i4,2)*elori(ielm)/(6.0d0*9.8d+03)	! ρI_x l/6/9800
        crl=emate(i4,5)*esect(i4,3)*elori(ielm)/(6.0d0*9.8d+03) ! ρI_y l/6/9800
        drl=brl+crl						! ρI_z l/6/9800
        do i=1,3
          emmat(i,i)=2.0d0*arl
          emmat(i,i+6)=arl
        enddo
        do i=7,9
          emmat(i,i-6)=arl
          emmat(i,i)=2.0d0*arl
        enddo
        emmat(4,4)=2.0d0*brl
        emmat(4,10)=brl
        emmat(10,4)=brl
        emmat(10,10)=2.0d0*brl
        emmat(5,5)=2.0d0*crl
        emmat(5,11)=crl
        emmat(11,5)=crl
        emmat(11,11)=2.0d0*crl
        emmat(6,6)=2.0d0*drl
        emmat(6,12)=drl
        emmat(12,6)=drl
        emmat(12,12)=2.0d0*drl
!
!*** 座標の変換 ***
      do i=1,12
        do j=1,12
          sum=0.0d0
          do k=1,12
            sum=sum+emmat(i,k)*ta(ielm,k,j)
          enddo
          emt(i,j)=sum
        enddo
      enddo
      do i=1,12
        do j=1,12
          sum=0.0d0
          do k=1,12
            sum=sum+ta(ielm,k,i)*emt(k,j)
          enddo
          emmat(i,j)=sum				! emmat=[T]^T [M_E][T]
        enddo
      enddo
!
      return
      end
!***********************************************************************
      subroutine elstif(ielm,ekmat,el,elori,emate,esect,ncr,ne,pmx,pmy,
     .                  pn,ss,ta,ts)
!*** 要素剛性マトリックス[K_E]の作成
!***********************************************************************
      include 'param.dat'
      dimension ekmat(mmtx,mmtx),el(melm),elori(melm),emate(mmate,mmd),
     .           esect(msect,msd),ncr(melm),ne(melm,mdim_ne),
     .           pmx(melm),pmy(melm),pn(melm),ss(melm),q(3),
     .           ta(melm,mmtx,mmtx),ts(melm,mdim),fd(3),
     .           bmatx(mdim,mmtx),db(mdim,mmtx),dmatx(mdim,mdim),
     .           ekb(mmtx,mmtx)
!
!*** マトリックス成分の初期化 ***
      do i=1,mmtx
        do j=1,mmtx
          ekb(i,j)=0.0d0
        enddo
      enddo
!
!*** [B]マトリックス，[D]マトリックスの読み込み ***
      s=ss(ielm)
      call bmake(ielm,s,bmatx,el,ta,esect,ne)
      call dmake(am,ielm,dmatx,emate,esect,fd,ncr,ne,pmx,pmy,pn,q,ts)
!
!*** 要素剛性マトリックス[K_E ]=[B]^T [D][B]lの計算 ***
      do i=1,6
        do j=1,12
          sum=0.0d0
          do k=1,6
            sum=sum+dmatx(i,k)*bmatx(k,j)
          enddo
          db(i,j)=sum
        enddo
      enddo
      do i=1,12
        do j=1,12
          sum=0.0d0
          do k=1,6
            sum=sum+bmatx(k,i)*db(k,j)
          enddo
          ekb(i,j)=sum
        enddo
      enddo
!
      do i=1,12
        do j=1,12
          ekmat(i,j)=ekb(i,j)*el(ielm)
        enddo
      enddo
!
      return
      end
!***********************************************************************
      subroutine force(istage,ndnod,nelm,nlstart,nlstep,ntdf,dlx,
     .                 dly,dlz,elori,emate,esect,f,ind,ne,nte,neg,
     .                 dltx,dlty,dltz)
!*** 外力の設定
!***********************************************************************
      include 'param.dat'
      dimension dlx(mnode),dly(mnode),dlz(mnode),elori(melm),
     .          emate(mmate,mmd),esect(msect,msd),f(mtdf),ftot(mtdf),
     .          ind(mnode),ne(melm,mdim_ne),nte(mtdf),
     .          dltx(mnode),dlty(mnode),dltz(mnode)
!
!*** パラメータの初期化 ***
      emass=0.0d0
      do i=1,mtdf
        ftot(i)=0.0d0
      enddo
!
!*** istage=0で重力作用下における自重を計算 ***
      if(istage==0) then
        do ielm=1,nelm
          i4=ne(ielm,4)
          if(neg==1) emass=-emate(i4,5)*esect(i4,1)*elori(ielm)/2.0d0  ! emass=-ρAl/2
          ftot(6*ne(ielm,1)-3)=ftot(6*ne(ielm,1)-3)+emass
          ftot(6*ne(ielm,2)-3)=ftot(6*ne(ielm,2)-3)+emass
        enddo
      endif
!
!*** 自由度ごとの外力増分の計算 *** 
      if(nlstart<=istage.and.istage<nlstart+nlstep) then
        do i=1,ndnod
          ftot(6*ind(i)-5)=dlx(ind(i))/dble(nlstep)
          ftot(6*ind(i)-4)=dly(ind(i))/dble(nlstep)
          ftot(6*ind(i)-3)=dlz(ind(i))/dble(nlstep)
          ftot(6*ind(i)-2)=dltx(ind(i))/dble(nlstep)
          ftot(6*ind(i)-1)=dlty(ind(i))/dble(nlstep)
          ftot(6*ind(i))=dltz(ind(i))/dble(nlstep)
        enddo
      endif
!
!*** 自由度番号の組み換え ***
      do i=1,ntdf
        i1=nte(i)
        f(i)=ftot(i1)
      enddo
!
      return
      end
!***********************************************************************
      subroutine gmake(ielm,nut,el,elori,gmatx,ta)
!*** 有限変形項を考慮する場合の[G]マトリックスの作成
!***********************************************************************
      include 'param.dat'
      dimension el(melm),elori(melm),gmatx(2,mmtx),ta(melm,mmtx,mmtx),
     .          ga(2,mmtx)
!
        do i=1,2
          do j=1,12
            ga(i,j)=0.0d0
          enddo
        enddo
!*** Ｇマトリクスの作成 ***
      if(nut==1) then !T.L.F.の場合
        ga(1,1)=-1.0d0/elori(ielm)
      elseif(nut==2) then !U.L.F.の場合
        ga(1,1)=-1.0d0/el(ielm)
      endif
      ga(1,7)=-ga(1,1)
      ga(2,2)=ga(1,1)
      ga(2,8)=ga(1,7)
!*** 座標変換 ***
      do i=1,2
        do j=1,12
          sum=0.0d0
          do k=1,12
            sum=sum+ga(i,k)*ta(ielm,k,j)
          enddo
          gmatx(i,j)=sum 			!gmatx=[G]×[T]
        enddo
      enddo
!
      return
      end
!***********************************************************************
      subroutine inicon(fout,tacc,te,ts,tsc,tsg,tsl,tsr,tu,tuout,tvel,x)
!*** パラメータの初期化
!***********************************************************************
      include 'param.dat'
      dimension fout(mtdf),tacc(mtdf),te(melm,mdim),tvel(mtdf),
     .          ts(melm,mdim),tsc(melm,mdim),tsg(melm,mdim),x(maxlena),
     .          tsl(melm,mdim),tsr(melm,mdim),tu(mtdf),tuout(mnode,3)
!
!*** 変数と配列の初期化 ***
      do i=1,mtdf
        fout(i)=0.0d0
        tu(i)=0.0d0
        tvel(i)=0.0d0
        tacc(i)=0.0d0
      enddo
      do i=1,mnode
        do j=1,3
          tuout(i,j)=0.0d0
        enddo
      enddo
      do i=1,maxlena
        x(i)=0.0d0
      enddo
      do i=1,melm
        do j=1,mdim
          te(i,j)=0.0d0
          ts(i,j)=0.0d0
          tsc(i,j)=0.0d0
          tsg(i,j)=0.0d0
          tsl(i,j)=0.0d0
          tsr(i,j)=0.0d0
        enddo
      enddo
!
      return
      end
!***********************************************************************
      subroutine input(beta_n,dalfa,dbeta,delta_n,dtime,inielm,nc,
     .                 ndnod,nelm,nfree,nlstart,nlstep,nmate,nmetd,
     .                 nmm,node,nout,nsd,nsect,nstage,ntdf,coord,
     .                 dlx,dly,dlz,emate,esect,ind,ir,mc,ne,pmx,pmy,
     .                 pn,ss,tuout,nstr,neg,nerr,neps,dltx,dlty,dltz)
!*** 入力データの読み込み
!***********************************************************************
      include 'param.dat'
      dimension coord(mnode+melm,3),dlx(mnode),dly(mnode),dlz(mnode),
     .          emate(mmate,mmd),esect(msect,msd),ind(mnode),ir(melm),
     .          mc(mnc,2),ne(melm,mdim_ne),pmx(melm),pmy(melm),pn(melm),
     .          ss(melm),tuout(mnode,3),temp(melm,11),dltx(mnode),
     .          dlty(mnode),dltz(mnode)
!
!*** 入力データファイルの先頭へ ***
      rewind ii5
!
!*** 基本情報の読み込み ***
      read(ii5,*) nelm,node,nstage,nout
      read(ii5,*) nmetd,nut,nif,nep,nmm,nsd,nstr,neg,nerr,neps
      read(ii5,*) beta_n,delta_n,dtime
        dalfa=0.0d0				! レーリー減衰はこのコードでは非考慮
        dbeta=0.0d0				! レーリー減衰はこのコードでは非考慮
        inielm=nelm				! 初期の全要素数を保存
        ntdf=6*node				! 全自由度数の計算
      if(nstage/=0.and.nout>nstage) nout=nstage
      if(nstage==0.or.nout==0) nout=1
!
!*** 基本情報の出力 ***
      write(ii9,900)
  900 format('<< STRUCTURAL ANALYSIS OF FRAMED STRUCTURE >>')
      write(ii9,901)
  901 format(/,'<< NUMERICAL CONDITIONS >>')
      write(ii9,902) nep
  902 format(/,'Elasto-plasticity =',i6,5x,
     .         '(1:Elastic, 2:Elasto-plastic)')
      write(ii9,903) nif
  903 format('Deformation theory =',i5,5x,'(1:Infinitesimal, 2:Finite)')
      write(ii9,904) nut
  904 format('Incremental theory =',i5,5x,'(1:TLF, 2:ULF)')
      write(ii9,905) nsd
  905 format('Static or Dynamic =',i6,5x,'(1:Static, 2:Dynamic)')
      write(ii9,906) nstr
  906 format('Structure =',i14,5x,'(1:Truss, 2:Rahmen)')
      write(ii9,907) nmetd
  907 format('Method =',i17,5x,
     .       '(1:Conventional scheme, 2:ASI, 3:ASI-Gauss)')
      write(ii9,942) nerr
  942 format('Output =',i17,5x,
     .       '(0:Normal mode, 1:Error message output mode)')
      write(ii9,943) neps
  943 format('Precision =',i14,5x,
     .       '(0:High, 1:Medium high, 2:Medium, 3:Low)')
!
      if(nsd==2) then				! 動的解析なら以下を出力
      write(ii9,908)
  908 format(/,'<< DYNAMIC PARAMETERS >>')
      write(ii9,909) nmm
  909 format(/,'Mass matrix =',i12,5x,'(1:Consistent, 2:Lumped)')
      write(ii9,910) beta_n,delta_n,dtime
  910 format('Newmark β =',f15.3,/,'Newmark δ =',f15.3,
     .       /,'Time increments [s] =',d12.5)
      endif
! 						! 出力条件
      write(ii9,911)
  911 format(/,'<< OUTPUT CONDITIONS >>')
      write(ii9,912) nstage,nout
  912 format(/,'No. of steps =',i11,/,'No. of outputs =',i9)
!
!*** 数値積分点と応力評価点の位置を初期化 ***
      do ielm=1,nelm
        if(nmetd==3) then 			! ASI-Gauss法
          ss(ielm)=1.0d0-2.0d0/dsqrt(3.0d0)
        else 					! 通常法とASI法
          ss(ielm)=0.0d0
        endif
        ir(ielm)=0
      enddo
!
!*** 外力条件の読み込み ***
      read(ii5,*) nlstep,ndnod
      nlstart=1
      do i=1,ndnod
      read(ii5,*) ind(i),dlx(ind(i)),dly(ind(i)),dlz(ind(i)),
     .            dltx(ind(i)),dlty(ind(i)),dltz(ind(i))
        dlx(ind(i))=dlx(ind(i))/9.8d0		! Nからkgfへ変更
        dly(ind(i))=dly(ind(i))/9.8d0
        dlz(ind(i))=dlz(ind(i))/9.8d0
        dltx(ind(i))=dltx(ind(i))/9.8d0
        dlty(ind(i))=dlty(ind(i))/9.8d0
        dltz(ind(i))=dltz(ind(i))/9.8d0
      enddo
!
!*** 外力条件の出力 ***
      write(ii9,936)
  936 format(/,'<< LOAD CONDITIONS >>')
      write(ii9,937) nlstart,nlstep,ndnod
  937 format(/,'Initial step to start loading =',i5,
     .       /,'Loading steps =',i21,/,'No. of nodes in loading =',i11)
      write(ii9,938) neg
  938 format('Dead load due to gravity =',i10,5x,
     .       '(0:No gravity, 1:Considering gravity)')
      write(ii9,939)
  939 format(/,'Node No.',3x,'Force(X) [N]',3x,'Force(Y) [N]',3x,
     .       'Force(Z) [N]',2x,'Moment(X)[Nmm]',1x,'Moment(Y)[Nmm]',1x,
     .       'Moment(Z)[Nmm]')
      do i=1,ndnod
      write(ii9,940) ind(i),dlx(ind(i))*9.8d0,dly(ind(i))*9.8d0,
     .               dlz(ind(i))*9.8d0,dltx(ind(i))*9.8d0,
     .               dlty(ind(i))*9.8d0,dltz(ind(i))*9.8d0
      enddo
  940 format(i4,3x,6d15.5)
!
!*** 材料物性値および断面形状パラメータの読み込み ***
      read(ii5,*) nmate
      nsect=nmate
      read(ii5,*) ((temp(i,j),j=1,11),i=1,nmate)
      do i=1,nmate
        emate(i,1)=temp(i,1)/9.8d0		! Nからkgfへ変更
        emate(i,2)=temp(i,2)
        emate(i,3)=temp(i,3)/9.8d0
        emate(i,4)=temp(i,4)/9.8d0
        emate(i,5)=temp(i,5)
      enddo
      do i=1,nsect
        esect(i,1)=temp(i,6)
        esect(i,2)=temp(i,7)
        esect(i,3)=temp(i,8)
        esect(i,4)=temp(i,9)
        esect(i,5)=temp(i,10)
        esect(i,6)=temp(i,11)
        esect(i,7)=5.0d0/6.0d0
        esect(i,8)=5.0d0/6.0d0
      enddo
!
!*** 材料物性値および断面形状パラメータの出力 ***
      write(ii9,913)
  913 format(/,'<< MEMBER CONDITIONS >>')
      write(ii9,914)
  914 format(/,'-- Material properties --')
      write(ii9,915) nmate
  915 format('No. of materials =',i7)
      write(ii9,916)
  916 format('Material No.',3x,'E [MPa]',10x,'ν',
     .       10x,'σy [MPa]',7x,'Et [MPa]',4x,'ρ [kg/mm^3]')
      do i=1,nmate
      write(ii9,917) i,emate(i,1)*9.8d0,emate(i,2),emate(i,3)*9.8d0,
     .               emate(i,4)*9.8d0,emate(i,5)
  917 format(i8,5d15.5)
      enddo
      write(ii9,918)
  918 format(/,'-- Sectional properties --')
      write(ii9,919) nsect
  919 format('No. of sections =',i8)
      write(ii9,920)
  920 format('Section No.',4x,'A [mm^2]',6x,'Ix [mm^4]',
     .       6x,'Iy [mm^4]',6x,'J [mm^4]',6x,'Zpx [mm^3]',
     .       5x,'Zpy [mm^3]')
      do i=1,nsect
      write(ii9,921) i,(esect(i,j),j=1,6)
  921 format(i8,6d15.5)
      enddo
!
!*** 拘束条件の読み込み ***
      read(ii5,*) nc
      read(ii5,*) ((mc(i,j),j=1,2),i=1,nc)
      nfree=ntdf-nc
!
!*** 全要素数，全節点数，拘束条件の出力 ***
      write(ii9,922)
  922 format(/,'<< MODEL CONDITIONS >>')
      write(ii9,923) nelm,node
  923 format(/,'No. of elements =',i8,/,'No. of nodes =',i11)
      write(ii9,932)
  932 format(/,'-- Fixed degrees of freedom --')
      write(ii9,933) nc
  933 format('No. of fixed degrees =',i5)
      write(ii9,934)
  934 format('Fixed degree No.',2x,'Node No.',2x,'Degree No.')
      do i=1,nc
      write(ii9,935) i,(mc(i,j),j=1,2)
  935 format(i4,8x,2i10)
      enddo
!
!*** 結合データの読み込み ***
      read(ii5,*) ((temp(ielm,j),j=1,5),ielm=1,nelm)
      do ielm=1,nelm
        do j=1,4
          ne(ielm,j)=temp(ielm,j)
        enddo
          ne(ielm,5)=1
          ne(ielm,6)=temp(ielm,5)
      enddo
!
!*** 結合データの出力 ***
      write(ii9,929)
  929 format(/,'-- Node connections --')
      write(ii9,930)
  930 format('Element No.',7x,'Connected nodes',6x,
     .       'Ref.node',3x,'Mat.Sect.No.',2x,'Pair element')
      do i=1,nelm
      write(ii9,931) i,(ne(i,j),j=1,4),ne(i,6)
  931 format(i4,4x,5i12)
      enddo
!
!*** 全断面が塑性化する際の断面力の算出 ***
      do ielm=1,nelm
        i4=ne(ielm,4)
        G=emate(i4,1)/(2.0d0*(1.0d0+emate(i4,2)))
        pmx(ielm)=emate(i4,3)*esect(i4,5)	! x軸回りの全塑性モーメント(=σ_y Z_px)
        pmy(ielm)=emate(i4,3)*esect(i4,6)	! y軸回りの全塑性モーメント(=σ_y Z_py)
        pn(ielm)=emate(i4,3)*esect(i4,1)  	! 降伏軸力(=σ_y A)
        if(nep==1) then				! 弾性解析では大きな値を設定
          pmx(ielm)=pmx(ielm)*1.0d+10
          pmy(ielm)=pmy(ielm)*1.0d+10
          pn(ielm)=pn(ielm)*1.0d+10
        endif
      enddo
!
!*** 節点および参考節点の初期座標値の読み込み ***
      do inode=1,node+nelm
        read(ii5,*) i,(coord(i,j),j=1,3)
      enddo
      do i=1,node				! 現ステップの節点座標値にも保存
        do j=1,3
          tuout(i,j)=coord(i,j) 
        enddo
      enddo
!
!*** 節点および参考節点の初期座標値の出力 ***
      write(ii9,924)
  924 format(/,'-- Node coordinates --')
      write(ii9,925)
  925 format('Node No.',7x,'X [mm]',9x,'Y [mm]',9x,'Z [mm]')
      do i=1,node
      write(ii9,926) i,(coord(i,j),j=1,3)
  926 format(i4,3x,3d15.5)
      enddo
      write(ii9,927)
  927 format('---------------------------------------- Reference nodes')
      do i=node+1,node+nelm
      write(ii9,926) i,(coord(i,j),j=1,3)
      enddo
      write(ii9,928)
  928 format('--------------------------------------------------------')
      write(ii9,941)
  941 format(/,'********************************************************
     .**************************************')
!
!*** 作成した解析モデルの形状を確認するために可視化データを出力 ***
      if(nerr==1) then
      write(ii6,'(2i8,x,f12.5)') inielm,node,-1.0
      write(ii6,'(12i7)') ((ne(ielm,j),j=1,2),ielm=1,inielm)
      do inode=1,node
        write(ii6,'(i7,6e13.5)') inode,(coord(inode,j),j=1,3)
      enddo
      do ielm=1,inielm
        write(ii7,*) 0.0,0.0
      enddo
      endif
!
      return
      end
!***********************************************************************
      subroutine output(inielm,iout,istage,node,nout,nstage,time,
     .                  eqsout,ne,tuout,tu,ts,nelm)
!*** 計算結果の出力
!***********************************************************************
      include 'param.dat'
      dimension eqsout(melm,2),ne(melm,mdim_ne),tuout(mnode,3),tu(mtdf),
     .          ts(melm,mdim)
!
      if(istage==0) iout=0
      if(nstage/=0.and.mod(istage,nstage/nout)/=0) return
!
!*** ポスト処理に必要な情報の出力 ***
      iout=iout+1
      write(ii6,'(2i8,x,f12.5)') inielm,node,time		! 要素数，節点数，時間
      write(ii6,'(12i7)') ((ne(ielm,j),j=1,2),ielm=1,inielm) 	! 結合データ
      do inode=1,node
        write(ii6,'(i7,6e13.5)') inode,(tuout(inode,j),j=1,3) 	! 座標値
      enddo
      do ielm=1,inielm
        write(ii7,*) (eqsout(ielm,j),j=1,2) 			! 降伏関数値
      enddo
      write(*,'(i7,3x,f10.3,3x,i5)') istage,time,iout
!
!*** 現ステップの番号と経過時間の出力 ***
      if(istage==0) write(ii9,941)
  941 format(/,'********************************************************
     .**************************************')
      if(istage==0) write(ii9,909)
      write(ii9,910) istage,time
  909 format('<< OUTPUTS >>')
  910 format(/2x,8histage =,i10,3x,8h  time =,e15.7,3x)
!
!*** 変位の出力 ***
      write(ii9,*) '-- Displacements --'
      write(ii9,919)
      do 200 i=1,node
      write(ii9,920) i,tu(6*i-5),tu(6*i-4),tu(6*i-3)
  200 continue
  919 format('Node No.',5x,'X [mm]',10x,'Y [mm]',10x,'Z [mm]')
  920 format(i4,3d16.4)
!
!*** 断面力の出力 ***
      write(ii9,*) '-- Sectional forces --'
      write(ii9,929)
      do 300 i=1,nelm
      write(ii9,930) i,(ts(i,j)*9.8d0,j=1,6)
  300 continue
  929 format('Element No.',1x,'Mx [Nmm]',8x,'My [Nmm]',9x,
     .       'N [N]',10x,'Mt [Nmm]',9x,'Vx [N]',10x,'Vy [N]')
  930 format(i4,6d16.4)
!
      return
      end
!***********************************************************************
      subroutine resid(nelm,nfree,ntdf,el,f,fout,ne,net,ss,ta,ts,esect)
!*** 外力，内力および残差力の計算
!***********************************************************************
      include 'param.dat'
      dimension el(melm),f(mtdf),fout(mtdf),ne(melm,mdim_ne),net(mtdf),
     .          ss(melm),ta(melm,mmtx,mmtx),ts(melm,mdim),
     .          bmatx(mdim,mmtx),btdg(mmtx),fint(mtdf),fres(mtdf),
     .          esect(msect,msd)
!
!*** 内力，残差力ベクトルの初期化 ***
      do i=1,ntdf
        fint(i)=0.0d0
        fres(i)=0.0d0
      enddo
!
!*** 内力の計算 ***
      do ielm=1,nelm
        s=ss(ielm)
      call bmake(ielm,s,bmatx,el,ta,esect,ne)
        do j=1,12
          sum=0.0d0
          do k=1,6
            sum=sum+bmatx(k,j)*ts(ielm,k)
          enddo
          btdg(j)=sum*el(ielm)
        enddo
        kevab=0
        do i=1,2
          nposn=6*(ne(ielm,i)-1)
          do idofn=1,6
            kevab=kevab+1
            nposn=nposn+1
            l1=net(nposn)
            if(l1>nfree) cycle
            fint(l1)=fint(l1)+btdg(kevab)
          enddo
        enddo
      enddo
!
!*** 外力，残差力および荷重増分の計算 ***
      do i=1,nfree
        fres(i)=fout(i)-fint(i)
        fout(i)=fout(i)+f(i)
        f(i)=f(i)+fres(i) 				! 残差力の荷重増分への加算
      enddo
!
      return
      end
!***********************************************************************
      subroutine scont(inielm,nelm,nmetd,ir,ncr,ne,ss,ts,tsc,
     .                 tsg,tsl,tsr)
!*** 数値積分点と応力評価点のシフティング
!***********************************************************************
      include 'param.dat'
      dimension ir(melm),ncr(melm),ne(melm,mdim_ne),ss(melm),
     .          ts(melm,mdim),tsc(melm,mdim),tsg(melm,mdim),
     .          tsl(melm,mdim),tsr(melm,mdim)
!
!*** 数値積分点と応力評価点のシフティング ***
      if(nmetd==1) then				! 通常法
        do ielm=1,nelm
          ss(ielm)=0.0d0
          do j=1,6
            ts(ielm,j)=tsc(ielm,j) 		! 要素中央点での値を断面力として使用
          enddo
        enddo
        return
      else 					! ASI法とASI-Gauss法
        do ielm=1,inielm
          if(ncr(ielm)>0) then 			! 塑性域にある要素
            ss(ielm)=dble(-ir(ielm))
            do j=1,6
              if(ir(ielm)==-1) ts(ielm,j)=tsl(ielm,j) 	! 始点での値を断面力として使用
              if(ir(ielm)==1) ts(ielm,j)=tsr(ielm,j) 	! 終点での値を断面力として使用
            enddo
            cycle
          elseif(ncr(ielm)<=0) then 			! 弾性域にある要素
            ss(ielm)=0.0d0				! 数値積分点は中央点に配置（ASI法）
            do j=1,6
              ts(ielm,j)=tsc(ielm,j) 			! 要素中央点での値を断面力として使用
            enddo
          endif
          if(nmetd==2) cycle			! ASI-Gauss法の場合
            i6=ne(ielm,6)
              if(ncr(ielm)<=0.and.ncr(i6)<=0) then 	
						! 対になる要素が両方とも弾性の場合
              ss(ielm)=1.0d0-2.0d0/dsqrt(3.0d0)	! 数値積分点の位置（図7.2参照）
              do j=1,6				
                ts(ielm,j)=tsg(ielm,j) 		! 部材のガウス積分点に相当する位置
              enddo				! での値を断面力として使用
            endif
        enddo
        do ielm=inielm+1,nelm			! 参考節点についての処理
          ss(ielm)=0.0d0			! 数値積分点は中央点に配置
          do j=1,6
            ts(ielm,j)=tsc(ielm,j)		! 要素中央点での値を断面力として使用
          enddo
        enddo
      endif
!
      return
      end
!***********************************************************************
      subroutine solve(beta_n,dalfa,dbeta,delta_n,dtime,istage,
     .                 nelm,nfree,nmetd,node,nsd,coord,el,emate,
     .                 eqs,eqsout,esect,f,ir,ncr,ne,net,nte,pmx,pmy,
     .                 pn,ss,t,ta,tacc,te,tru,ts,tsc,tsg,tsl,tsr,
     .                 tu,tuout,tvel,x,nstr,neg,elori,neps)
!*** 変位，ひずみおよび断面力の計算
!***********************************************************************
      include 'param.dat'
      dimension coord(mnode+melm,3),el(melm),emate(mmate,mmd),
     .           eqs(melm),eqsout(melm,2),esect(msect,msd),f(mtdf),
     .           ir(melm),ncr(melm),ne(melm,mdim_ne),net(mtdf),
     .           nte(mtdf),pmx(melm),pmy(melm),pn(melm),elori(melm),
     .           ss(melm),t(melm,3,3),ta(melm,mmtx,mmtx),tacc(mtdf),
     .           te(melm,mdim),tru(melm,3,3),ts(melm,mdim),
     .           tsc(melm,mdim),tsg(melm,mdim),tsl(melm,mdim),
     .           tsr(melm,mdim),tu(mtdf),tuout(mnode,3),tvel(mtdf),
     .           x(maxlena),bmatx(mdim,mmtx),de(mdim),dmatx(mdim,mdim),
     .           dul(mmtx),ds(mdim),fd(3),q(3),temp(mdim),tst(mdim,mdim)
!
      if(istage==0.or.nsd==1) then 		! 自重解析または静的解析
      if(istage==0.and.neg==0) goto 100 	! 重力作用無しなら自重解析をとばす
!
!*** CG法を用いて剛性方程式を解く ***
        call cdyna_first(nfree,f,x,neps)
  100 continue
      else
!
!*** CG法を用いて運動方程式を解く ***
        call cdyna(beta_n,dalfa,dbeta,delta_n,dtime,nfree,f,tacc,tvel,x,
     .             neps)
      endif
      do ielm=1,nelm
!
!*** 算出された変位を保存 ***
        do i=1,2
          do j=1,6
            i1=6*(ne(ielm,i)-1)+j
            temp(j)=x(net(i1))
          enddo
          do k=1,6
            dul(6*(i-1)+k)=temp(k)
          enddo
        enddo
!
!*** ひずみ増分と全ひずみの計算（(8.11)式参照）***
        s=ss(ielm)
      call bmake(ielm,s,bmatx,el,ta,esect,ne)
        do i=1,6
          sum=0.0d0
          do j=1,12
            sum=sum+bmatx(i,j)*dul(j)
          enddo
          de(i)=sum
          te(ielm,i)=te(ielm,i)+sum
        enddo
!
!*** 断面力増分の計算（(8.12)式参照）***
      call dmake(am,ielm,dmatx,emate,esect,fd,ncr,ne,pmx,pmy,pn,q,ts)
      call trans2(ielm,dul,el,t,tru,tst)
        do i=1,6
          sum=0.0d0
          do j=1,6
            sum=sum+dmatx(i,j)*de(j)
          enddo
          temp(i)=sum
        enddo
          do i=1,6
            sum=0.0d0
            do j=1,6
              sum=sum+tst(i,j)*temp(j)		! 更新Kirchhoff 断面力増分ベクトル
            enddo				! からJaumann 微分形ベクトルへの変換
            ds(i)=sum
          enddo
!
!*** 要素内の各位置での断面力の計算（(7.11)式と(7.19)式参照）***
        if(nstr==1) then 			! トラス構造の場合
          j=3
          tsc(ielm,j)=tsc(ielm,j)+ds(j)
          tsg(ielm,j)=tsg(ielm,j)+ds(j)
          tsl(ielm,j)=tsl(ielm,j)+ds(j)
          tsr(ielm,j)=tsr(ielm,j)+ds(j)
        else 					! ラーメン構造の場合
          do j=3,6
            tsc(ielm,j)=tsc(ielm,j)+ds(j)
            tsg(ielm,j)=tsg(ielm,j)+ds(j)
            tsl(ielm,j)=tsl(ielm,j)+ds(j)
            tsr(ielm,j)=tsr(ielm,j)+ds(j)
          enddo
          gauss=2.0d0/dsqrt(3.0d0)-1.0d0 	! 弾性域での応力評価点の位置
          if(idnint(s*10.0d0)==0) then		! 数値積分点がs=0にある場合
            gc=gauss/2.0d0
            do j=1,2				! 各位置での断面力の足し込み
              tsc(ielm,j)=tsc(ielm,j)+ds(j)
              tsg(ielm,j)=tsg(ielm,j)+ds(j)-ds(7-j)*el(ielm)*gc
              tsl(ielm,j)=tsl(ielm,j)+ds(j)+ds(7-j)*el(ielm)*0.5d0
              tsr(ielm,j)=tsr(ielm,j)+ds(j)-ds(7-j)*el(ielm)*0.5d0
            enddo
          elseif(idnint(s*10.0d0)==-10) then	! 数値積分点がs=-1にある場合
            gr=0.5d0-gauss/2.0d0
            do j=1,2				! 各位置での断面力の足し込み
              tsr(ielm,j)=tsr(ielm,j)+ds(j)
              tsc(ielm,j)=tsc(ielm,j)+ds(j)+ds(7-j)*el(ielm)*0.5d0
              tsg(ielm,j)=tsg(ielm,j)+ds(j)+ds(7-j)*el(ielm)*gr
              tsl(ielm,j)=tsl(ielm,j)+ds(j)+ds(7-j)*el(ielm)
            enddo
          elseif(idnint(s*10.0d0)==10) then	! 数値積分点がs=1にある場合
            gl=0.5d0+gauss/2.0d0
            do j=1,2				! 各位置での断面力の足し込み
              tsl(ielm,j)=tsl(ielm,j)+ds(j)
              tsc(ielm,j)=tsc(ielm,j)+ds(j)-ds(7-j)*el(ielm)*0.5d0
              tsg(ielm,j)=tsg(ielm,j)+ds(j)-ds(7-j)*el(ielm)*gl
              tsr(ielm,j)=tsr(ielm,j)+ds(j)-ds(7-j)*el(ielm)
            enddo
          elseif(idint(s*10.0d0)==-1) then	! 数値積分点がs=s_gにある場合
            gc=gauss/2.0d0
            gl=0.5d0+gauss/2.0d0
            gr=0.5d0-gauss/2.0d0
            do j=1,2				! 各位置での断面力の足し込み
              tsg(ielm,j)=tsg(ielm,j)+ds(j)
              tsc(ielm,j)=tsc(ielm,j)+ds(j)+ds(7-j)*el(ielm)*gc
              tsl(ielm,j)=tsl(ielm,j)+ds(j)+ds(7-j)*el(ielm)*gl
              tsr(ielm,j)=tsr(ielm,j)+ds(j)-ds(7-j)*el(ielm)*gr
            enddo
          else					! それ以外の場合は計算を中止
          write(ii9,'(21hss(ielm) ERROR: ielm=,i5,4h  s=,e11.4)') ielm,s
          stop
          endif
        endif
!
!*** 降伏関数値の計算（(6.28)式参照）***
        if(nmetd==1) then 			! 通常法の場合
          eqs(ielm)=(tsc(ielm,1)/pmx(ielm))*(tsc(ielm,1)/pmx(ielm))
     .              +(tsc(ielm,2)/pmy(ielm))*(tsc(ielm,2)/pmy(ielm))
     .              +(tsc(ielm,3)/pn(ielm))*(tsc(ielm,3)/pn(ielm))
          eqsout(ielm,1)=eqs(ielm) 		! 可視化用データ（要素中央点での値）
          eqsout(ielm,2)=eqs(ielm)
          if(eqsout(ielm,1)>1.0d0) eqsout(ielm,1)=1.0d0
          if(eqsout(ielm,2)>1.0d0) eqsout(ielm,2)=1.0d0
        else 					! ASI法とASI-Gauss法の場合
          eqsl=(tsl(ielm,1)/pmx(ielm))*(tsl(ielm,1)/pmx(ielm))
     .         +(tsl(ielm,2)/pmy(ielm))*(tsl(ielm,2)/pmy(ielm))
     .         +(tsl(ielm,3)/pn(ielm))*(tsl(ielm,3)/pn(ielm))
 						! 要素始点（部材中央点）での値
          eqsr=(tsr(ielm,1)/pmx(ielm))*(tsr(ielm,1)/pmx(ielm))
     .         +(tsr(ielm,2)/pmy(ielm))*(tsr(ielm,2)/pmy(ielm))
     .         +(tsr(ielm,3)/pn(ielm))*(tsr(ielm,3)/pn(ielm)) 
 						! 要素終点（部材端）での値
          eqsout(ielm,1)=eqsl 			! 可視化用データ（要素始点での値）
          eqsout(ielm,2)=eqsr			! 可視化用データ（要素終点での値）
          if(eqsl>1.0d0) eqsout(ielm,1)=1.0d0
          if(eqsr>1.0d0) eqsout(ielm,2)=1.0d0
          if(ncr(ielm)<=0) then 		! 弾性域ではeqsとしてeqslとeqsrの
            ir(ielm)=0				! 大きい方の値を使用
            if(eqsl>eqsr) then
              eqs(ielm)=eqsl
              if(eqsl>=1.0d0) ir(ielm)=-1	! 始点で降伏なら応力評価点を始点へ
            elseif(eqsr>=eqsl) then
              eqs(ielm)=eqsr
              if(eqsr>=1.0d0) ir(ielm)=1	! 終点で降伏なら応力評価点を終点へ
            endif
          elseif(ncr(ielm)>0) then 		! 要素が塑性域にある場合
            if(ir(ielm)==-1) eqs(ielm)=eqsl	! 応力評価点が始点にあるなら始点のfyを使用
            if(ir(ielm)==1) eqs(ielm)=eqsr	! 応力評価点が終点にあるなら終点のfyを使用
          endif
        endif
!
!*** 弾性/塑性化/除荷のクラス分け ***
        if(ncr(ielm)<=0) then 			! 要素が弾性域にある場合
          if(eqs(ielm)>=1.0d0) then 		
	    ncr(ielm)=1				! もしf_y>1.0なら要素は塑性化(ncr=1)
          else
            if(ncr(ielm)<0) ncr(ielm)=-2 	! 除荷が継続している場合(ncr=-2)
            cycle
          endif
        elseif(ncr(ielm)>0) then 		! 要素が塑性域にある場合
          ul=0.0d0
          do i=1,3
            ul=ul+q(i)*de(i)
          enddo
          ul=ul/am
          if(ul<0.0d0) ncr(ielm)=-1 		! 現ステップで除荷した場合 (ncr=-1)
          if(ul>=0.0d0) ncr(ielm)=2 		! 塑性化が継続している場合(ncr=2)
        endif
        do j=1,6 				! 塑性理論に従い断面力を降下
          tsc(ielm,j)=tsc(ielm,j)/dsqrt(eqs(ielm))
          tsg(ielm,j)=tsg(ielm,j)/dsqrt(eqs(ielm))
          tsl(ielm,j)=tsl(ielm,j)/dsqrt(eqs(ielm))
          tsr(ielm,j)=tsr(ielm,j)/dsqrt(eqs(ielm))
        enddo
        eqs(ielm)=1.0d0
      enddo
!
!*** 変位，節点座標と要素長の計算 ***
      do i=1,nfree
        tu(nte(i))=tu(nte(i))+x(i)		! 変位に変位増分の足し込み
      enddo
      do inode=1,node
        do j=1,3
          tuout(inode,j)=coord(inode,j)+tu(6*(inode-1)+j) ! 座標値＝初期座標値＋変位
        enddo
      enddo
      do ielm=1,nelm
        xx=tuout(ne(ielm,2),1)-tuout(ne(ielm,1),1)
        yy=tuout(ne(ielm,2),2)-tuout(ne(ielm,1),2)
        zz=tuout(ne(ielm,2),3)-tuout(ne(ielm,1),3)
        el(ielm)=dsqrt(xx*xx+yy*yy+zz*zz)	! 現ステップでの要素長の計算
      enddo
!
      return
      end
!***********************************************************************
      subroutine tmasm(nelm,nmm,nfree,ntdf,elori,emate,esect,ne,ta)
!*** 全体質量マトリックスの作成
!***********************************************************************
      include 'param.dat'
      dimension elori(melm),emate(mmate,mmd),esect(msect,msd),
     .           ne(melm,mdim_ne),ta(melm,mmtx,mmtx),
     .           emmat(mmtx,mmtx)
!
      call gminit(nfree,ntdf-nfree)
!
!*** 要素質量マトリックスと全体質量マトリックスの作成 ***
      do ielm=1,nelm
        call elmass(ielm,nmm,elori,emate,emmat,esect,ne,ta)
        call insmmat(ielm,emmat,ke,icon)
      enddo
!
      return
      end
!***********************************************************************
      subroutine trans1(istage,nelm,coord,el,elori,ne,t,ta,tru,nerr)
!*** 座標変換マトリックスの作成
!***********************************************************************
      include 'param.dat'
      dimension coord(mnode+melm,3),el(melm),elori(melm),
     .          ne(melm,mdim_ne),t(melm,3,3),ta(melm,mmtx,mmtx),
     .          tru(melm,3,3),dji(3),dki(3),temp(3,3)
!
      if(istage.eq.0.and.nerr.eq.1) 
     . write(ii9,*) '-- Searching for input errors --'
!
      if(istage.eq.0) then
        do ielm=1,nelm
          ip=ne(ielm,1)
          jp=ne(ielm,2)
          kp=ne(ielm,3)
!
      if(nerr.eq.1) write(ii9,*) 'Element No. = ',ielm
!
!*** 初期要素長の計算 ***
          xx=coord(jp,1)-coord(ip,1)
          yy=coord(jp,2)-coord(ip,2)
          zz=coord(jp,3)-coord(ip,3)
          el(ielm)=dsqrt(xx*xx+yy*yy+zz*zz)
          elori(ielm)=el(ielm)
!
!*** 参考節点を用いた初期の座標変換マトリックスの導出 ***
          do j=1,3
            dji(j)=coord(jp,j)-coord(ip,j) 		! 方向ベクトルの計算
            dki(j)=coord(kp,j)-coord(ip,j)
          enddo
          vp1=dji(2)*dki(3)-dji(3)*dki(2)		! 外積
          vp2=dji(3)*dki(1)-dji(1)*dki(3)
          vp3=dji(1)*dki(2)-dji(2)*dki(1)
          vp=dsqrt(vp1*vp1+vp2*vp2+vp3*vp3)
          t(ielm,3,1)=dji(1)/elori(ielm)
          t(ielm,3,2)=dji(2)/elori(ielm)
          t(ielm,3,3)=dji(3)/elori(ielm)
          t(ielm,2,1)=vp1/vp
          t(ielm,2,2)=vp2/vp
          t(ielm,2,3)=vp3/vp
          t(ielm,1,1)=t(ielm,2,2)*t(ielm,3,3)-t(ielm,2,3)*t(ielm,3,2)
          t(ielm,1,2)=t(ielm,2,3)*t(ielm,3,1)-t(ielm,2,1)*t(ielm,3,3)
          t(ielm,1,3)=t(ielm,2,1)*t(ielm,3,2)-t(ielm,2,2)*t(ielm,3,1)
          do j=1,mmtx
            do k=1,mmtx
             ta(ielm,j,k)=0.0d0
            enddo
          enddo
        if(nerr.eq.1) write(ii9,*) ' O.K.'
        enddo
!
!*** ULFを用いた更新座標変換マトリックスの作成（(8.6)式参照）***
      elseif(istage>0) then
        do ielm=1,nelm
          do i=1,3
            do j=1,3
              temp(i,j)=t(ielm,i,j)
            enddo
          enddo
          do i=1,3
            do j=1,3
              sum=0.0d0
              do k=1,3
                sum=sum+tru(ielm,i,k)*temp(k,j)
              enddo
              t(ielm,i,j)=sum
            enddo
          enddo
        enddo
      endif
!*** 12自由度のための座標変換マトリックスの作成（(8.8)式参照）***
      do ielm=1,nelm
        do j=1,3
          do k=1,3
            ta(ielm,j,k)=t(ielm,j,k)
            ta(ielm,j+3,k+3)=t(ielm,j,k)
            ta(ielm,j+6,k+6)=t(ielm,j,k)
            ta(ielm,j+9,k+9)=t(ielm,j,k)
          enddo
        enddo
      enddo
!
      return
      end
!***********************************************************************
      subroutine trans2(ielm,dul,el,t,tru,tst)
!*** ULFのための座標変換マトリックスの作成
!***********************************************************************
      include 'param.dat'
      dimension dul(mmtx),el(melm),t(melm,3,3),tru(melm,3,3),
     .          tst(mdim,mdim),du(3)
!
!*** 要素座標系における相対節点変位増分の計算 ***
      do i=1,3
        sum=0.0d0
        do j=1,3
          sum=sum+t(ielm,i,j)*(dul(j+6)-dul(j))
        enddo
        du(i)=sum
      enddo
!
!*** sinα, cosα, sinβ, cosβ, sinγとcosγの計算（(8.10)式参照）***
      dz=el(ielm)+du(3)
      sina=-du(2)/dsqrt(dz*dz+du(2)*du(2))
      cosa=dz/dsqrt(dz*dz+du(2)*du(2))
      sinb=du(1)/dsqrt(dz*dz+du(2)*du(2)+du(1)*du(1))
      cosb=dsqrt(dz*dz+du(2)*du(2))/dsqrt(dz*dz+du(2)*du(2)+du(1)*du(1))
      sum=0.0d0
      do j=1,3
        sum=sum+t(ielm,3,j)*(dul(j+3)+dul(j+9))
      enddo
      cosc=dcos(sum/2.0d0)
      sinc=dsin(sum/2.0d0)
!
!*** 座標変換マトリックスの計算（(8.9)式参照）***
      tru(ielm,1,1)=cosb*cosc
      tru(ielm,1,2)=sina*sinb*cosc+cosa*sinc
      tru(ielm,1,3)=-cosa*sinb*cosc+sina*sinc
      tru(ielm,2,1)=-cosb*sinc
      tru(ielm,2,2)=-sina*sinb*sinc+cosa*cosc
      tru(ielm,2,3)=cosa*sinb*sinc+sina*cosc
      tru(ielm,3,1)=sinb
      tru(ielm,3,2)=-sina*cosb
      tru(ielm,3,3)=cosa*cosb
!
!*** 更新Kirchhoff 断面力増分ベクトルからJaumann 微分形ベクトルへの座標変換マトリックスの
!　　作成（(8.14)式参照）***
      do i=1,mdim
        do j=1,mdim
          tst(i,j)=0.0d0
        enddo
      enddo
      tst(1,1)=tru(ielm,1,1)
      tst(1,2)=tru(ielm,1,2)
      tst(1,4)=tru(ielm,1,3)
      tst(2,1)=tru(ielm,2,1)
      tst(2,2)=tru(ielm,2,2)
      tst(2,4)=tru(ielm,2,3)
      tst(3,3)=tru(ielm,3,3)
      tst(3,5)=tru(ielm,3,1)
      tst(3,6)=tru(ielm,3,2)
      tst(4,1)=tru(ielm,3,1)
      tst(4,2)=tru(ielm,3,2)
      tst(4,4)=tru(ielm,3,3)
      tst(5,3)=tru(ielm,1,3)
      tst(5,5)=tru(ielm,1,1)
      tst(5,6)=tru(ielm,1,2)
      tst(6,3)=tru(ielm,2,3)
      tst(6,5)=tru(ielm,2,1)
      tst(6,6)=tru(ielm,2,2)
!
      return
      end
!***********************************************************************
      subroutine tstfns(nelm,nfree,ntdf,el,elori,emate,esect,ncr,ne,
     .                  pmx,pmy,pn,ss,ta,ts)
!*** 全体剛性マトリックスの作成
!***********************************************************************
      include 'param.dat'
      dimension el(melm),elori(melm),emate(mmate,mmd),esect(msect,msd),
     .          ncr(melm),ne(melm,mdim_ne),pmx(melm),pmy(melm),
     .          pn(melm),ss(melm),ta(melm,mmtx,mmtx),ts(melm,mdim),
     .          ekmat(mmtx,mmtx)
!
!*** 全体剛性マトリックスの初期化 ***
      call gkinit(nfree,ntdf-nfree)
!
!*** 要素剛性マトリックスと全体剛性マトリックスの作成 ***
      do ielm=1,nelm
        call elstif(ielm,ekmat,el,elori,emate,esect,ncr,ne,pmx,pmy,pn,
     .              ss,ta,ts)
        call inskmat(ielm,ekmat,ke,icon)
      enddo
!
      return
      end
!***********************************************************************
!**************************** CG METHOD ********************************
!***********************************************************************
!
!***********************************************************************
      subroutine adrchk(lena,lenb)
!*** CG method *** Calculate index of address
!***********************************************************************
      include 'param.dat'
      data lenk/0/
      save lk0
!
      nfr=lena+lenb
      ilm(1)=1
      do 132 ifr=2,nfr
  132  ilm(ifr)=ilm(ifr-1)+irmb(ifr-1)
      do 134 ifr=1,nfr
       irma(ifr)=irma(ifr)+ilm(ifr)-1
  134  irmb(ifr)=irmb(ifr)+ilm(ifr)-1
      if(lk0.ne.lenk) lk0=lenk
      if(irmb(nfr).gt.maxlenk) then
      write(ii9,*) 'lenk =',irmb(nfr),'/',maxlenk
      write(ii9,*) 'maxlenk over!!'
      stop
      endif
!
      return
      end
!***********************************************************************
      subroutine cdyna(bt,dalfa,dbeta,delta,dtime,nfree,f,tacc,
     .                 tvel,x,neps)
!*** CG法を用いた運動方程式の計算
!***********************************************************************
      include 'param.dat'
      dimension f(mtdf),tacc(mtdf),tvel(mtdf),x(maxlena),
     .          w1(maxlena),w2(maxlena),w3(maxlena),w4(maxlena)
!
!*** Newmarkのβ法（(8.4)式参照）***
!  [C]=α[M]+β[K], レーリー減衰マトリックス
      do i=1,nfree
        do j=ilm(i),irma(i)
          gkmat(j)=gkmat(j)+gmmat(j)/(bt*dtime*dtime)
     .             +(dalfa*gmmat(j)+dbeta*gkmat(j))*delta/(bt*dtime)
        enddo
      enddo
!
      do i=1,nfree
        j=ilm(i)
        f(i)=f(i)+gmmat(j)*tvel(jtab(j))/(bt*dtime)
     .       +gmmat(j)*tacc(jtab(j))*(1.0d0/(2.0d0*bt)-1.0d0)
     .       +(dalfa*gmmat(j)+dbeta*gkmat(j))
     .       *((delta/bt-1.0d0)*tvel(jtab(j))
     .       +(delta/(2.0d0*bt)-1.0d0)*dtime*tacc(jtab(j)))
        do j=ilm(i)+1,irma(i)
          f(i)=f(i)+gmmat(j)*tvel(jtab(j))/(bt*dtime)
     .         +gmmat(j)*tacc(jtab(j))*(1.0d0/(2.0d0*bt)-1.0d0)
     .         +(dalfa*gmmat(j)+dbeta*gkmat(j))
     .         *((delta/bt-1.0d0)*tvel(jtab(j))
     .         +(delta/(2.0d0*bt)-1.0d0)*dtime*tacc(jtab(j)))
          f(jtab(j))=f(jtab(j))+gmmat(j)*tvel(i)/(bt*dtime)
     .               +gmmat(j)*tacc(i)*(1.0d0/(2.0d0*bt)-1.0d0)
     .               +(dalfa*gmmat(j)+dbeta*gkmat(j))
     .               *((delta/bt-1.0d0)*tvel(i)
     .               +(delta/(2.0d0*bt)-1.0d0)*dtime*tacc(i))
        enddo
      enddo
!
!*** 反復計算のためのノルムの設定 ***
      if(neps==0) eps=1.0d-14
      if(neps==1) eps=1.0d-6
      if(neps==2) eps=1.0d-3
      if(neps==3) eps=1.0d-1
      imax=mtdf					! 反復回数を大きな数に設定
!
!*** CG法による連立方程式の計算 ***
      call ddpcg(gkmat,jtab,nfree,ilm,irma,f,x,imax,eps,nstop,
     .           w1,w2,w3,w4)
      if(nstop==2) then	! 収束しなかったら計算を止める
        write(*,'(/1x,36hThe solution is still not converged!)')
        write(ii9,'(/36hThe solution is still not converged!)')
        stop
      endif
!
!*** 加速度増分と速度増分（(8.2)式および(8.3)式参照）***
      do i=1,nfree
        dvel=x(i)*delta/(bt*dtime)-tvel(i)*delta/bt
     .       +(1.0d0-delta/(2.0d0*bt))*dtime*tacc(i)
        dacc=x(i)/(bt*dtime*dtime)-tvel(i)/(bt*dtime)-tacc(i)/(2.0d0*bt)
        tvel(i)=tvel(i)+dvel
        tacc(i)=tacc(i)+dacc
      enddo
!
      return
      end
!***********************************************************************
      subroutine cdyna_first(nfree,f,x,neps)
!*** CG法を用いた剛性方程式の計算
!***********************************************************************
      include 'param.dat'
      dimension f(mtdf),x(maxlena),w1(maxlena),w2(maxlena),w3(maxlena),
     .           w4(maxlena)
!
!*** 反復計算のためのノルムの設定 ***
      if(neps==0) eps=1.0d-14
      if(neps==1) eps=1.0d-6
      if(neps==2) eps=1.0d-3
      if(neps==3) eps=1.0d-1
      imax=mtdf					! 反復回数を大きな数に設定
!
!*** CG法による連立方程式の計算 ***
      call ddpcg(gkmat,jtab,nfree,ilm,irma,f,x,imax,eps,nstop,
     .           w1,w2,w3,w4)
      if(nstop==2) then				! 収束しなかったら計算を止める
        write(*,'(/1x,36hThe solution is still not converged!)')
        write(ii9,'(/36hThe solution is still not converged!)')
        stop
      endif
!
      return
      end
!***********************************************************************
      subroutine cginit(nelm,nfree,node,ntdf,ne,net)
!*** CG method *** Initialize variables
!***********************************************************************
      include 'param.dat'
      dimension ne(melm,mdim_ne),net(mtdf)
!
      do 110 ielm=1,nelm
       do 120 j=1,2
        ke(j,ielm)=ne(ielm,j)
  120  continue
  110 continue
      do 210 inode=1,node
       do 220 j=1,6
        icon(j,inode)=net(6*inode-6+j)
  220  continue
  210 continue
!
      call jtabset(nelm,ke,node,icon,nfree,ntdf-nfree)
!
      return
      end
!***********************************************************************
      subroutine connect(inode,nelem,melem,ncnct,mcnct)
!*** CG method *** Connecting node information
!***********************************************************************
      include 'param.dat'
      dimension melem(nonpe,*),mcnct(*)
      logical contain,newnode
!
      ncnct=0
      do 114 ielem=1,nelem
       contain= .false.
       do 110 inod=1,nonpe
       if(melem(inod,ielem).eq.inode) contain= .true.
  110  continue
!
!    If this element doesn't contain inode,the flow skips.
!
       if(.not. contain) go to 114
       do 115 inod=1,nonpe
        jnode=melem(inod,ielem)
        newnode= .true.
        do 112 icnct=1,ncnct
         if(jnode.eq.mcnct(icnct)) newnode= .false.
  112   continue
        if(newnode) then
        ncnct=ncnct+1
        mcnct(ncnct)=jnode
        endif
  115   continue
  114 continue
      return
      end
!***********************************************************************
      subroutine ddpcg(a,j,n,ilm,irm,b,x,imax,eps,nstop,r,p,ap,d)
!*** CG method *** Solver; diag-preconditioned conjugate gradient method
!***********************************************************************
      implicit Double Precision(a-h,o-z)
      dimension a(*),j(*),ilm(*),irm(*),b(*),x(*),r(*),p(*),ap(*),d(*)
!
!    Param check
!
      if(imax.lt.0.or.n.lt.2) go to 93
      imax=min0((3*n)/2,imax)
!
!    Diag-precondition
!
      do 1 i=1,n
      atemp=a(ilm(i))
    1 d(i)=1.0d0/dsqrt(atemp)
      do 2 i=1,n
       b(i)=b(i)*d(i)
       r(i)=b(i)
    2  x(i)=x(i)/d(i)
      do 3 i=1,n
       do 3 k=ilm(i),irm(i)
    3  a(k)=a(k)*d(i)*d(j(k))
!
!    Initialization
!
      do 13 i=1,n
       do 11 k=ilm(i),irm(i)
   11  r(i)=r(i)-a(k)*x(j(k))
       do 12 k=ilm(i)+1,irm(i)
   12  r(j(k))=r(j(k))-a(k)*x(i)
   13 continue
      do 14 i=1,n
   14 p(i)=r(i)
      bnorm=0.0d0
      do 19 i=1,n
   19 bnorm=bnorm+b(i)*b(i)
      tor=bnorm*eps*eps
      iter=0
!
!    Alpha=(p,r)/(p,ap)
!
   21 iter=iter+1
      do 22 i=1,n
   22 ap(i)=0.0d0
      do 25 i=1,n
       do 23 k=ilm(i),irm(i)
   23  ap(i)=ap(i)+a(k)*p(j(k))
       do 24 k=ilm(i)+1,irm(i)
   24  ap(j(k))=ap(j(k))+a(k)*p(i)
   25 continue
      pxr=0.0d0
      pxap=0.0d0
      do 26 i=1,n
      pxr=pxr+p(i)*r(i)
   26 pxap=pxap+p(i)*ap(i)
      alpha=pxr/pxap
!
!    X=x+alpha*p, r=r-alpha*ap
!
      do 31 i=1,n
   31 x(i)=x(i)+alpha*p(i)
      do 41 i=1,n
   41 r(i)=r(i)-alpha*ap(i)
      rnorm=0.0d0
      do 42 i=1,n
   42 rnorm=rnorm+r(i)*r(i)
      if(rnorm.lt.tor) go to 91
!
!    Beta=-(r,ap)/(p,ap), p=r+beta*p
!
      rxap=0.0d0
      do 51 i=1,n
   51 rxap=rxap+r(i)*ap(i)
      beta=-rxap/pxap
      do 61 i=1,n
   61 p(i)=r(i)+beta*p(i)
      if(iter.le.imax) go to 21
      imax=iter
      nstop=2
      go to 92
!
   91 nstop=1
   92 continue
      do 71 i=1,n
71    x(i)=x(i)*d(i)
      return
   93 nstop=3
      return
      end
!***********************************************************************
      subroutine gkinit(lena,lenb)
!*** CG method *** Initialize total stiffness matrix
!***********************************************************************
      include 'param.dat'
!
      do 12 i=1,irmb(lena+lenb)
   12 gkmat(i)=0.0d0
!
      return
      end
!***********************************************************************
      subroutine gminit(lena,lenb)
!*** CG method *** Initialize total mass matrix
!***********************************************************************
      include 'param.dat'
!
      do 12 i=1,irmb(lena+lenb)
   12 gmmat(i)=0.0d0
!
      return
      end
!***********************************************************************
      subroutine inskmat(ielem,ekmat,melem,ifropt)
!*** CG method *** Construction of total stiffness matrix
!***********************************************************************
      include 'param.dat'
      dimension ekmat(mmtx,mmtx),melem(nonpe,*),ifropt(nofpn,*),
     .          idx(noef),ip(noef)
!
      do 112 inod=1,nonpe
       do 112 icoord=1,nofpn
  112  idx((inod-1)*nofpn+icoord)
     +    =ifropt(icoord,melem(inod,ielem))
      do 124 iren=noef,1,-1
       idxmax=0
       do 122 j=1,noef
       if(idx(j).gt.idxmax) then
       idxmax=idx(j)
       nmax=j
       endif
  122  continue
       idx(nmax)=-idx(nmax)
       ip(iren)=nmax
  124 continue
      do 126 i=1,noef
  126 idx(i)=-idx(i)
!
      do 132 i=1,noef
      k=ilm(idx(i))-1
       do 132 iren=1,noef
       j=ip(iren)
       if(idx(j).lt.idx(i)) go to 132
  130   k=k+1
        if(idx(j).ne.jtab(k)) go to 130
        gkmat(k)=gkmat(k)+ekmat(i,j)
  132 continue
      return
      end
!***********************************************************************
      subroutine insmmat(ielem,emmat,melem,ifropt)
!*** CG method *** Construction of total mass matrix
!***********************************************************************
      include 'param.dat'
      dimension emmat(mmtx,mmtx),melem(nonpe,*),ifropt(nofpn,*),
     .          idx(noef),ip(noef)
!
      do 112 inod=1,nonpe
       do 112 icoord=1,nofpn
  112  idx((inod-1)*nofpn+icoord)
     +    =ifropt(icoord,melem(inod,ielem))
      do 124 iren=noef,1,-1
      idxmax=0
       do 122 j=1,noef
       if(idx(j).gt.idxmax) then
       idxmax=idx(j)
       nmax=j
       endif
  122  continue
      idx(nmax)=-idx(nmax)
      ip(iren)=nmax
  124 continue
      do 126 i=1,noef
  126 idx(i)=-idx(i)
!
      do 132 i=1,noef
      k=ilm(idx(i))-1
       do 132 iren=1,noef
       j=ip(iren)
       if(idx(j).lt.idx(i)) go to 132
  130  k=k+1
       if(idx(j).ne.jtab(k)) go to 130
       gmmat(k)=gmmat(k)+emmat(i,j)
  132 continue
      return
      end
!***********************************************************************
      subroutine isort1(idata,n)
!*** CG method *** Logical sortation of freedom number
!***********************************************************************
      dimension idata(*)
      logical sorted
!
      k=n
   10 if(k.le.1) return
      k=k-1
      sorted= .true.
      do 20 i=1,k
      if(idata(i).le.idata(i+1)) go to 20
      l=idata(i)
      idata(i)=idata(i+1)
      idata(i+1)=l
      sorted= .false.
   20 continue
      if(sorted) return
      go to 10
      end
!***********************************************************************
      subroutine jtabset(nelem,melem,knode,ifropt,lena,lenb)
!*** CG method *** Pre-task of CG method
!***********************************************************************
      include 'param.dat'
      dimension melem(nonpe,*),ifropt(nofpn,*),mcnct(maxnode),
     .          jtabr(maxfr)
      data la0,lb0/0,0/
      save la0,lb0
!
      if(la0.ne.lena.or.lb0.ne.lenb) then
       la0=lena
       lb0=lenb
      endif
      if(lena.gt.maxlena.or.lenb.gt.maxlenb) then
        write(ii9,*) 'lena =',lena,'/',maxlena
        write(ii9,*) 'lenb =',lenb,'/',maxlenb
        write(ii9,*) 'maxlena or maxlenb over!!'
        stop
      endif
!     open(unit=ii4,status='scratch',form='unformatted',file='tmp.sp')
      open(unit=ii4,form='unformatted')
      rewind(ii4)
      do 120 inode=1,knode
!
! Nodes connecting to 'inode' are given.
!
      call connect(inode,nelem,melem,ncnct,mcnct)
!
! If j is greater than or equals to i, number of j is set.
!
      do 120 icoord=1,nofpn
      ifr=ifropt(icoord,inode)
      lenra=0
      lenrb=0
      do 124 icnct=1,ncnct
       do 124 jcord=1,nofpn
       jfr=ifropt(jcord,mcnct(icnct))
       if(jfr.ge.ifr) then
       lenrb=lenrb+1
       jtabr(lenrb)=jfr
       if(jfr.le.lena) lenra=lenra+1
       endif
  124 continue
!
! Array "jtabr" is sorted.
!
      call isort1(jtabr,lenrb)
      irma(ifr)=lenra
      irmb(ifr)=lenrb
      write(ii4) (jtabr(i),i=1,lenrb)
  120 continue
      call adrchk(lena,lenb)
      rewind(ii4)
      do 140 inode=1,knode
       do 140 icoord=1,nofpn
       ifr=ifropt(icoord,inode)
       read(ii4) (jtab(i),i=ilm(ifr),irmb(ifr))
  140 continue
      close(ii4)
      return
      end